@@ -2,10 +2,17 @@ from __future__ import absolute_import, division, print_function
 
 from _echopype_version import version as __version__  # noqa
 
-from . import calibrate
+from . import calibrate, preprocess, visualize
 from .convert.api import open_raw
 from .echodata.api import open_converted
 from .echodata.combine import combine_echodata
 from .process import Process  # noqa
 
-__all__ = ["open_raw", "open_converted", "combine_echodata", "calibrate"]
+__all__ = [
+    "open_raw",
+    "open_converted",
+    "combine_echodata",
+    "calibrate",
+    "visualize",
+    "preprocess",
+]
