@@ -1,5 +1,6 @@
 import numpy as np
 import xarray as xr
+from dask.array.core import Array
 from scipy import signal
 
 from ..utils import uwa
@@ -14,16 +15,26 @@ class CalibrateEK(CalibrateBase):
         self.env_params = dict.fromkeys(ENV_PARAMS)
         self.cal_params = dict.fromkeys(CAL_PARAMS["EK"])
 
-    def compute_range_meter(self, waveform_mode, encode_mode="complex"):
+    def compute_range_meter(self, waveform_mode, encode_mode):
         """
         Parameters
         ----------
-        waveform_mode : str
-            - ``CW`` for CW-mode samples, either recorded as complex or power samples
-            - ``BB`` for BB-mode samples, recorded as complex samples
-        tvg_correction_factor : int
-            - 2 for CW-mode power samples
-            - 0 for CW-mode complex samples
+        waveform_mode : {"CW", "BB"}
+            Type of transmit waveform.
+            Required only for data from the EK80 echosounder.
+
+            - `"CW"` for narrowband transmission,
+              returned echoes recorded either as complex or power/angle samples
+            - `"BB"` for broadband transmission,
+              returned echoes recorded as complex samples
+
+        encode_mode : {"complex", "power"}
+            Type of encoded return echo data.
+            Required only for data from the EK80 echosounder.
+
+            - `"complex"` for complex samples
+            - `"power"` for power/angle samples, only allowed when
+              the echosounder is configured for narrowband transmission
 
         Returns
         -------
@@ -36,7 +47,7 @@ class CalibrateEK(CalibrateBase):
             ek_encode_mode=encode_mode,
         )
 
-    def _get_vend_cal_params_power(self, param):
+    def _get_vend_cal_params_power(self, param, waveform_mode):
         """Get cal parameters stored in the Vendor group.
 
         Parameters
@@ -70,12 +81,12 @@ class CalibrateEK(CalibrateBase):
                 dim="ping_time", how="any", subset=["transmit_duration_nominal"]
             )
 
-        if self.echodata.beam_power is not None:
+        if waveform_mode == "CW" and self.echodata.beam_power is not None:
             beam = self.echodata.beam_power
         else:
             beam = self.echodata.beam
 
-            # indexes of frequencies that are for power, not complex
+        # indexes of frequencies that are for power, not complex
         relevant_indexes = np.where(np.isin(ds_vend["frequency"], beam["frequency"]))[0]
 
         unique_pulse_length = np.unique(beam["transmit_duration_nominal"], axis=1)
@@ -87,13 +98,17 @@ class CalibrateEK(CalibrateBase):
             dim="pulse_length_bin"
         )
 
+        # Checks for dask array and compute first
+        if isinstance(idx_wanted.data, Array):
+            idx_wanted = idx_wanted.data.compute()
+
         return (
             ds_vend[param]
             .isel(pulse_length_bin=idx_wanted, frequency=relevant_indexes)
             .drop("pulse_length_bin")
         )
 
-    def get_cal_params(self, cal_params, waveform_mode=None, encode_mode="complex"):
+    def get_cal_params(self, cal_params, waveform_mode, encode_mode):
         """Get cal params using user inputs or values from data file.
 
         Parameters
@@ -121,7 +136,7 @@ class CalibrateEK(CalibrateBase):
                 self.cal_params[p] = (
                     cal_params[p]
                     if p in cal_params
-                    else self._get_vend_cal_params_power(p)
+                    else self._get_vend_cal_params_power(p, waveform_mode=waveform_mode)
                 )
 
         # Other params
@@ -131,7 +146,7 @@ class CalibrateEK(CalibrateBase):
             else beam["equivalent_beam_angle"]
         )
 
-    def _cal_power(self, cal_type, use_beam_power=False):
+    def _cal_power(self, cal_type, use_beam_power=False) -> xr.Dataset:
         """Calibrate power data from EK60 and EK80.
 
         Parameters
@@ -146,7 +161,8 @@ class CalibrateEK(CalibrateBase):
 
         Returns
         -------
-        Sv or Sp
+        xr.Dataset
+            The calibrated dataset contaning Sv or Sp
         """
         # Select source of backscatter data
         if use_beam_power:
@@ -304,7 +320,7 @@ class CalibrateEK80(CalibrateEK):
         # self.range_meter computed under self._compute_cal()
         # because the implementation is different depending on waveform_mode and encode_mode
 
-    def get_env_params(self, env_params, waveform_mode=None, encode_mode="complex"):
+    def get_env_params(self, env_params, waveform_mode="BB", encode_mode="complex"):
         """Get env params using user inputs or values from data file.
 
         EK80 file by default contains sound speed, temperature, depth, salinity, and acidity,
@@ -315,13 +331,21 @@ class CalibrateEK80(CalibrateEK):
         Parameters
         ----------
         env_params : dict
-        waveform_mode : str
-            ``CW`` for CW-mode samples, either recorded as complex or power samples
-            ``BB`` for BB-mode samples, recorded as complex samples
-        encode_mode : str
-            EK80 data can be encoded as complex samples or power samples.
-            Use ``complex`` to compute Sv from only complex samples,
-            and ``power`` to compute Sv from only power samples.
+
+        waveform_mode : {"CW", "BB"}
+            Type of transmit waveform.
+
+            - `"CW"` for narrowband transmission,
+              returned echoes recorded either as complex or power/angle samples
+            - (default) `"BB"` for broadband transmission,
+              returned echoes recorded as complex samples
+
+        encode_mode : {"complex", "power"}
+            Type of encoded return echo data.
+
+            - (default) `"complex"` for complex samples
+            - `"power"` for power/angle samples, only allowed when
+              the echosounder is configured for narrowband transmission
         """
 
         if (
@@ -563,21 +587,23 @@ class CalibrateEK80(CalibrateEK):
 
         return y_all, y_time_all, tau_effective
 
-    def compress_pulse(self, chirp):
+    def compress_pulse(self, chirp, freq_BB=None):
         """Perform pulse compression on the backscatter data.
 
         Parameters
         ----------
         chirp : dict
             transmit chirp replica indexed by channel_id
+        freq_BB : int or float
+            frequency channels that transmit in BB mode
+            (since CW mode can be in mixed in complex samples too)
         """
-        backscatter = (
-            self.echodata.beam["backscatter_r"]
-            + 1j * self.echodata.beam["backscatter_i"]
-        )
+        backscatter = self.echodata.beam["backscatter_r"].sel(
+            frequency=freq_BB
+        ) + 1j * self.echodata.beam["backscatter_i"].sel(frequency=freq_BB)
 
         pc_all = []
-        for freq in self.echodata.beam.frequency.values:
+        for freq in freq_BB:
             backscatter_freq = (
                 backscatter.sel(frequency=freq)
                 .dropna(dim="range_bin", how="all")
@@ -604,7 +630,69 @@ class CalibrateEK80(CalibrateEK):
 
         return pc_merge
 
-    def _cal_complex(self, cal_type, waveform_mode):
+    def _get_gain_for_complex(self, waveform_mode, freq_center) -> xr.DataArray:
+        """Get gain factor for calibrating complex samples.
+
+        Use values from ``gain_correction`` in the Vendor group for CW mode samples,
+        or interpolate ``gain`` to the center frequency of each ping for BB mode samples
+        if nominal frequency is within the calibrated frequencies range
+
+        Parameters
+        ----------
+        waveform_mode : str
+            ``CW`` for CW-mode samples, either recorded as complex or power samples
+            ``BB`` for BB-mode samples, recorded as complex samples
+        freq_center : xr.DataArray
+            Nominal channel frequency for CW mode samples
+            and an xr.DataArray with coorindate ``frequency`` and ``ping_time`` for BB mode samples
+
+        Returns
+        -------
+        An xr.DataArray
+        """
+        if waveform_mode == "BB":
+            gain_single = self._get_vend_cal_params_power(
+                "gain_correction", waveform_mode=waveform_mode
+            )
+            gain = []
+            if "gain" in self.echodata.vendor.data_vars:
+                # index using channel_id as order of frequency across channel can be arbitrary
+                # refererence to freq_center in case some channels are CW complex samples
+                # (already dropped when computing freq_center in the calling function)
+                for fn in freq_center.frequency:
+                    ch_id = self.echodata.beam.channel_id.sel(frequency=fn)
+                    # if freq-dependent gain exists in data
+                    if ch_id in self.echodata.vendor.cal_channel_id:
+                        gain_vec = self.echodata.vendor.gain.sel(cal_channel_id=ch_id)
+                        gain_temp = (
+                            gain_vec.interp(
+                                cal_frequency=freq_center.sel(frequency=fn)
+                            ).drop(["cal_channel_id", "cal_frequency"])
+                        ).expand_dims("frequency")
+                    # if no freq-dependent gain use CW gain
+                    else:
+                        gain_temp = (
+                            gain_single.sel(frequency=fn)
+                            .assign_coords(ping_time=np.datetime64(0, "ns"))
+                            .expand_dims("ping_time")
+                            .reindex_like(
+                                self.echodata.beam.backscatter_r, method="nearest"
+                            )
+                            .expand_dims("frequency")
+                        )
+                    gain_temp.name = "gain"
+                    gain.append(gain_temp)
+                gain = xr.merge(gain).gain  # select the single data variable
+            else:
+                gain = gain_single
+        elif waveform_mode == "CW":
+            gain = self._get_vend_cal_params_power(
+                "gain_correction", waveform_mode=waveform_mode
+            ).sel(frequency=freq_center.frequency)
+
+        return gain
+
+    def _cal_complex(self, cal_type, waveform_mode) -> xr.Dataset:
         """Calibrate complex data from EK80.
 
         Parameters
@@ -612,16 +700,47 @@ class CalibrateEK80(CalibrateEK):
         cal_type : str
             'Sv' for calculating volume backscattering strength, or
             'Sp' for calculating point backscattering strength
-        waveform_mode : str
-            ``CW`` for CW-mode samples, either recorded as complex or power samples
-            ``BB`` for BB-mode samples, recorded as complex samples
+        waveform_mode : {"CW", "BB"}
+            Type of transmit waveform.
+
+            - `"CW"` for narrowband transmission,
+              returned echoes recorded either as complex or power/angle samples
+            - `"BB"` for broadband transmission,
+              returned echoes recorded as complex samples
+
+        Returns
+        -------
+        xr.Dataset
+            The calibrated dataset contaning Sv or Sp
         """
         # Transmit replica and effective pulse length
         chirp, _, tau_effective = self.get_transmit_chirp(waveform_mode=waveform_mode)
 
-        # pulse compression
+        # use center frequency for each ping to select BB or CW channels
+        # when all samples are encoded as complex samples
+        if (
+            "frequency_start" in self.echodata.beam
+            and "frequency_end" in self.echodata.beam
+        ):
+            freq_center = (
+                self.echodata.beam["frequency_start"]
+                + self.echodata.beam["frequency_end"]
+            ) / 2
+        else:
+            freq_center = None
+
         if waveform_mode == "BB":
-            pc = self.compress_pulse(chirp)
+            if freq_center is None:
+                raise ValueError(
+                    "frequency_start and frequency_end should exist in BB mode data, "
+                    "double check the EchoData object!"
+                )
+            # if CW and BB complex samples co-exist
+            # drop those that contain CW samples (nan in freq start/end)
+            freq_sel = freq_center.dropna(dim="frequency")
+
+            # backscatter data
+            pc = self.compress_pulse(chirp, freq_BB=freq_sel.frequency)
             prx = (
                 self.echodata.beam.quadrant.size
                 * np.abs(pc.mean(dim="quadrant")) ** 2
@@ -630,6 +749,15 @@ class CalibrateEK80(CalibrateEK):
                 / self.z_et
             )
         else:
+            if freq_center is None:
+                # when only have CW complex samples
+                freq_sel = self.echodata.beam.frequency
+            else:
+                # if BB and CW complex samples co-exist
+                # drop those that contain BB samples (not nan in freq start/end)
+                freq_sel = freq_center.where(np.isnan(freq_center), drop=True).frequency
+
+            # backscatter data
             backscatter_cw = (
                 self.echodata.beam["backscatter_r"]
                 + 1j * self.echodata.beam["backscatter_i"]
@@ -644,56 +772,66 @@ class CalibrateEK80(CalibrateEK):
             prx.name = "received_power"
             prx = prx.to_dataset()
 
-        # Derived params
+        # derived params
         sound_speed = self.env_params["sound_speed"].squeeze()
-        range_meter = self.range_meter
-        freq_nominal = self.echodata.beam.frequency
+        absorption = (
+            self.env_params["sound_absorption"]
+            .sel(frequency=freq_sel.frequency)
+            .squeeze()
+        )
+        range_meter = self.range_meter.sel(frequency=freq_sel.frequency).squeeze()
         if waveform_mode == "BB":
-            freq_center = (
-                self.echodata.beam["frequency_start"]
-                + self.echodata.beam["frequency_end"]
-            ) / 2
-            wavelength = sound_speed / freq_center
-        elif waveform_mode == "CW":
-            wavelength = sound_speed / freq_nominal
-        # gain = self.echodata.vendor['gain']  # TODO: need to interpolate gain to at freq_center
-        gain = 27
+            # use true center frequency for BB pulse
+            wavelength = sound_speed / freq_sel
+
+            # use true center frequency to interpolate for gain factor
+            gain = self._get_gain_for_complex(
+                waveform_mode=waveform_mode, freq_center=freq_sel
+            )
+
+        else:
+            # use nominal channel frequency for CW pulse
+            wavelength = sound_speed / freq_sel
+
+            # use nominal channel frequency to select gain factor
+            gain = self._get_gain_for_complex(
+                waveform_mode=waveform_mode, freq_center=freq_sel
+            )
 
         # Transmission loss
-        spreading_loss = (
-            20 * np.log10(range_meter.where(range_meter >= 1, other=1)).squeeze()
-        )
-        absorption_loss = (
-            2 * self.env_params["sound_absorption"].squeeze() * range_meter.squeeze()
-        )
+        spreading_loss = 20 * np.log10(range_meter.where(range_meter >= 1, other=1))
+        absorption_loss = 2 * absorption * range_meter
 
         # TODO: both Sv and Sp are off by ~<0.5 dB from matlab outputs.
         #  Is this due to the use of 'single' in matlab code?
         if cal_type == "Sv":
-            # get equivalent beam angle
-            if waveform_mode == "BB":
-                psifc = self.echodata.beam["equivalent_beam_angle"] + 10 * np.log10(
-                    freq_nominal / freq_center
-                )
-            elif waveform_mode == "CW":
-                psifc = self.echodata.beam["equivalent_beam_angle"]
-
             # effective pulse length
             tau_effective = xr.DataArray(
                 data=list(tau_effective.values()),
                 coords=[self.echodata.beam.frequency, self.echodata.beam.ping_time],
                 dims=["frequency", "ping_time"],
+            ).sel(frequency=freq_sel.frequency)
+
+            # other params
+            transmit_power = self.echodata.beam["transmit_power"].sel(
+                frequency=freq_sel.frequency
             )
+            if waveform_mode == "BB":
+                psifc = self.echodata.beam["equivalent_beam_angle"].sel(
+                    frequency=freq_sel.frequency
+                ) + 10 * np.log10(freq_sel.frequency / freq_center)
+            elif waveform_mode == "CW":
+                psifc = self.echodata.beam["equivalent_beam_angle"].sel(
+                    frequency=freq_sel.frequency
+                )
+
             out = (
                 10 * np.log10(prx)
                 + spreading_loss
                 + absorption_loss
                 - 10
                 * np.log10(
-                    wavelength ** 2
-                    * self.echodata.beam["transmit_power"]
-                    * sound_speed
-                    / (32 * np.pi ** 2)
+                    wavelength ** 2 * transmit_power * sound_speed / (32 * np.pi ** 2)
                 )
                 - 2 * gain
                 - 10 * np.log10(tau_effective)
@@ -702,16 +840,15 @@ class CalibrateEK80(CalibrateEK):
             out = out.rename_vars({list(out.data_vars.keys())[0]: "Sv"})
 
         elif cal_type == "Sp":
+            transmit_power = self.echodata.beam["transmit_power"].sel(
+                frequency=freq_sel.frequency
+            )
+
             out = (
                 10 * np.log10(prx)
                 + 2 * spreading_loss
                 + absorption_loss
-                - 10
-                * np.log10(
-                    wavelength ** 2
-                    * self.echodata.beam["transmit_power"]
-                    / (16 * np.pi ** 2)
-                )
+                - 10 * np.log10(wavelength ** 2 * transmit_power / (16 * np.pi ** 2))
                 - 2 * gain
             )
             out = out.rename_vars({list(out.data_vars.keys())[0]: "Sp"})
@@ -724,31 +861,47 @@ class CalibrateEK80(CalibrateEK):
 
         return out
 
-    def _compute_cal(self, cal_type, waveform_mode, encode_mode):
-        """Private method to compute Sv or Sp called by compute_Sv or compute_Sp.
+    def _compute_cal(self, cal_type, waveform_mode, encode_mode) -> xr.Dataset:
+        """
+        Private method to compute Sv or Sp from EK80 data, called by compute_Sv or compute_Sp.
 
         Parameters
         ----------
         cal_type : str
             'Sv' for calculating volume backscattering strength, or
             'Sp' for calculating point backscattering strength
-        waveform_mode : str
-            ``BB`` for BB-mode samples, recorded as complex samples (default)
-            ``CW`` for CW-mode samples, either recorded as complex or power samples
-        encode_mode : str
-            EK80 data can be encoded as complex samples or power samples.
-            Use ``complex`` to compute Sv from only complex samples,
-            and ``power`` to compute Sv from only power samples.
+
+        waveform_mode : {"CW", "BB"}
+            Type of transmit waveform.
+
+            - `"CW"` for narrowband transmission,
+              returned echoes recorded either as complex or power/angle samples
+            - `"BB"` for broadband transmission,
+              returned echoes recorded as complex samples
+
+        encode_mode : {"complex", "power"}
+            Type of encoded return echo data.
+
+            - `"complex"` for complex samples
+            - `"power"` for power/angle samples, only allowed when
+              the echosounder is configured for narrowband transmission
 
         Returns
         -------
-        Dataset containing either Sv or Sp.
+        xr.Dataset
+            An xarray Dataset containing either Sv or Sp.
         """
         # Raise error for wrong inputs
         if waveform_mode not in ("BB", "CW"):
-            raise ValueError("Input waveform_mode not recognized!")
+            raise ValueError(
+                "Input waveform_mode not recognized! "
+                "waveform_mode must be either 'BB' or 'CW' for EK80 data."
+            )
         if encode_mode not in ("complex", "power"):
-            raise ValueError("Input encode_mode not recognized!")
+            raise ValueError(
+                "Input encode_mode not recognized! "
+                "encode_mode must be either 'complex' or 'power' for EK80 data."
+            )
 
         # Set flag_complex
         #  - True: complex cal
@@ -760,13 +913,18 @@ class CalibrateEK80(CalibrateEK):
                     "encode_mode='power' not allowed when waveform_mode='BB'!"
                 )
             flag_complex = True
-        else:
+        else:  # waveform_mode="CW"
             if encode_mode == "complex":
                 flag_complex = True
             else:
                 flag_complex = False
-        # TODO: add additional checks and error messages for
-        #  when waveform_mode and actual recording mode do not match
+
+        # Raise error when waveform_mode and actual recording mode do not match
+        # This simple check is only possible for BB-only data,
+        #   since for data with both BB and CW complex samples,
+        #   frequency_start will exist in echodata.beam for the BB channels
+        if waveform_mode == "BB" and "frequency_start" not in self.echodata.beam:
+            raise ValueError("waveform_mode='BB' but broadband data not found!")
 
         # Set use_beam_power
         #  - True: use self.echodata.beam_power for cal
@@ -774,9 +932,16 @@ class CalibrateEK80(CalibrateEK):
         use_beam_power = False
 
         # Warn user about additional data in the raw file if another type exists
+        # When both power and complex samples exist:
+        #   complex samples will be stored in echodata.beam
+        #   power samples will be stored in echodata.beam_power
+        # When only one type of samples exist,
+        #   all samples with be stored in echodata.beam
         if self.echodata.beam_power is not None:  # both power and complex samples exist
-            if encode_mode == "complex" and waveform_mode == "CW":
-                raise ValueError("file does not contain CW complex samples")
+            # If both beam and beam_power groups exist,
+            #   this means that CW data are encoded as power samples and in beam_power group
+            if waveform_mode == "CW" and encode_mode == "complex":
+                raise ValueError("File does not contain CW complex samples")
 
             if encode_mode == "power":
                 use_beam_power = True  # switch source of backscatter data
@@ -793,21 +958,23 @@ class CalibrateEK80(CalibrateEK):
             ):  # data contain only complex samples
                 if encode_mode == "power":
                     raise TypeError(
-                        "File does not contain power samples!"
+                        "File does not contain power samples! Use encode_mode='complex'"
                     )  # user selects the wrong encode_mode
             else:  # data contain only power samples
                 if encode_mode == "complex":
                     raise TypeError(
-                        "File does not contain complex samples!"
+                        "File does not contain complex samples! Use encode_mode='power'"
                     )  # user selects the wrong encode_mode
 
         # Compute Sv
         if flag_complex:
+            # Complex samples can be BB or CW
             self.compute_range_meter(
                 waveform_mode=waveform_mode, encode_mode=encode_mode
             )
             ds_cal = self._cal_complex(cal_type=cal_type, waveform_mode=waveform_mode)
         else:
+            # Power samples only make sense for CW mode data
             self.compute_range_meter(waveform_mode="CW", encode_mode=encode_mode)
             ds_cal = self._cal_power(cal_type=cal_type, use_beam_power=use_beam_power)
 
@@ -818,15 +985,20 @@ class CalibrateEK80(CalibrateEK):
 
         Parameters
         ----------
-        encode_mode : str
-            For EK80 data by default calibration is performed for the complex samples.
-            Use ``complex`` to compute Sv from only complex samples (default),
-            and ``power`` to compute Sv from only power samples.
-            Note if waveform_mode='BB', only complex samples are collected.
-            For all other sonar systems, calibration for power samples is performed.
-        waveform_mode : str
-            ``BB`` for BB-mode samples, recorded as complex samples (default)
-            ``CW`` for CW-mode samples, either recorded as complex or power samples
+        waveform_mode : {"CW", "BB"}
+            Type of transmit waveform.
+
+            - `"CW"` for narrowband transmission,
+              returned echoes recorded either as complex or power/angle samples
+            - (default) `"BB"` for broadband transmission,
+              returned echoes recorded as complex samples
+
+        encode_mode : {"complex", "power"}
+            Type of encoded return echo data.
+
+            - (default) `"complex"` for complex samples
+            - `"power"` for power/angle samples, only allowed when
+              the echosounder is configured for narrowband transmission
 
         Returns
         -------
@@ -843,15 +1015,20 @@ class CalibrateEK80(CalibrateEK):
 
         Parameters
         ----------
-        encode_mode : str
-            For EK80 data by default calibration is performed for the complex samples.
-            Use ``complex`` to compute Sv from only complex samples (default),
-            and ``power`` to compute Sv from only power samples.
-            Note if waveform_mode='BB', only complex samples are collected.
-            For all other sonar systems, calibration for power samples is performed.
-        waveform_mode : str
-            ``BB`` for BB-mode samples, recorded as complex samples (default)
-            ``CW`` for CW-mode samples, either recorded as complex or power samples
+        waveform_mode : {"CW", "BB"}
+            Type of transmit waveform.
+
+            - `"CW"` for narrowband transmission,
+              returned echoes recorded either as complex or power/angle samples
+            - (default) `"BB"` for broadband transmission,
+              returned echoes recorded as complex samples
+
+        encode_mode : {"complex", "power"}
+            Type of encoded return echo data.
+
+            - (default) `"complex"` for complex samples
+            - `"power"` for power/angle samples, only allowed when
+              the echosounder is configured for narrowband transmission
 
         Returns
         -------
