@@ -125,7 +125,9 @@ class ParseEK(ParseBase):
             # Rectangularize all data and convert to numpy array indexed by channel
             for data_type in ["power", "angle", "complex"]:
                 for k, v in self.ping_data_dict[data_type].items():
-                    if all(x is None for x in v):  # if no data in a particular channel
+                    if all(
+                        (x is None) or (x.size == 0) for x in v
+                    ):  # if no data in a particular channel
                         self.ping_data_dict[data_type][k] = None
                     else:
                         # Sort complex and power/angle channels
@@ -347,13 +349,14 @@ class ParseEK(ParseBase):
         if (
             np.unique(lens).size != 1
         ):  # if some pings have different lengths along range
+
             if data_list[0].ndim == 2:
                 # Angle data have an extra dimension for alongship and athwartship samples
                 mask = lens[:, None, None] > np.array([np.arange(lens.max())] * 2).T
             else:
                 mask = lens[:, None] > np.arange(lens.max())
             # Take care of problem of np.nan being implicitly "real"
-            if isinstance(data_list[0][0], (np.complex, np.complex64, np.complex128)):
+            if data_list[0].dtype in {np.dtype("complex64"), np.dtype("complex128")}:
                 out_array = np.full(mask.shape, np.nan + 0j)
             else:
                 out_array = np.full(mask.shape, np.nan)
