@@ -18,7 +18,7 @@ import numpy as np
 
 from .ek_date_conversion import nt_to_unix
 
-TCVR_CH_NUM_MATCHER = re.compile(r"\d{6}-\w{1,2}")
+TCVR_CH_NUM_MATCHER = re.compile(r"\d{6}-\w{1,2}|\w{12}-\w{1,2}")
 
 __all__ = [
     "SimradNMEAParser",
@@ -416,7 +416,7 @@ class SimradNMEAParser(_SimradDatagramParser):
                              ready for writing to disk
     """
 
-    nmea_head_re = re.compile("\$[A-Za-z]{5},")  # noqa
+    nmea_head_re = re.compile(r"\$[A-Za-z]{5},")  # noqa
 
     def __init__(self):
         headers = {
@@ -1151,7 +1151,7 @@ class SimradFILParser(_SimradDatagramParser):
 
 class SimradConfigParser(_SimradDatagramParser):
     """
-    Simrad Configuration Datagram parser operates on dictonaries with the following keys:
+    Simrad Configuration Datagram parser operates on dictionaries with the following keys:
 
         type:         string == 'CON0'
         low_date:     long uint representing LSBytes of 64bit NT date
@@ -1176,7 +1176,7 @@ class SimradConfigParser(_SimradDatagramParser):
         beam_config                     [str] Raw XML string containing beam config. info
 
 
-    Transducer Config Keys (ER60/ES60 sounders):
+    Transducer Config Keys (ER60/ES60/ES70 sounders):
         channel_id                      [str]   channel ident string
         beam_type                       [long]  Type of channel (0 = Single, 1 = Split)
         frequency                       [float] channel frequency
@@ -1238,6 +1238,34 @@ class SimradConfigParser(_SimradDatagramParser):
                         ready for writing to disk
     """
 
+    COMMON_KEYS = [
+        ("channel_id", "128s"),
+        ("beam_type", "l"),
+        ("frequency", "f"),
+        ("gain", "f"),
+        ("equivalent_beam_angle", "f"),
+        ("beamwidth_alongship", "f"),
+        ("beamwidth_athwartship", "f"),
+        ("angle_sensitivity_alongship", "f"),
+        ("angle_sensitivity_athwartship", "f"),
+        ("angle_offset_alongship", "f"),
+        ("angle_offset_athwartship", "f"),
+        ("pos_x", "f"),
+        ("pos_y", "f"),
+        ("pos_z", "f"),
+        ("dir_x", "f"),
+        ("dir_y", "f"),
+        ("dir_z", "f"),
+        ("pulse_length_table", "5f"),
+        ("spare1", "8s"),
+        ("gain_table", "5f"),
+        ("spare2", "8s"),
+        ("sa_correction_table", "5f"),
+        ("spare3", "8s"),
+        ("gpt_software_version", "16s"),
+        ("spare4", "28s"),
+    ]
+
     def __init__(self):
         headers = {
             0: [
@@ -1257,60 +1285,9 @@ class SimradConfigParser(_SimradDatagramParser):
         _SimradDatagramParser.__init__(self, "CON", headers)
 
         self._transducer_headers = {
-            "ER60": [
-                ("channel_id", "128s"),
-                ("beam_type", "l"),
-                ("frequency", "f"),
-                ("gain", "f"),
-                ("equivalent_beam_angle", "f"),
-                ("beamwidth_alongship", "f"),
-                ("beamwidth_athwartship", "f"),
-                ("angle_sensitivity_alongship", "f"),
-                ("angle_sensitivity_athwartship", "f"),
-                ("angle_offset_alongship", "f"),
-                ("angle_offset_athwartship", "f"),
-                ("pos_x", "f"),
-                ("pos_y", "f"),
-                ("pos_z", "f"),
-                ("dir_x", "f"),
-                ("dir_y", "f"),
-                ("dir_z", "f"),
-                ("pulse_length_table", "5f"),
-                ("spare1", "8s"),
-                ("gain_table", "5f"),
-                ("spare2", "8s"),
-                ("sa_correction_table", "5f"),
-                ("spare3", "8s"),
-                ("gpt_software_version", "16s"),
-                ("spare4", "28s"),
-            ],
-            "ES60": [
-                ("channel_id", "128s"),
-                ("beam_type", "l"),
-                ("frequency", "f"),
-                ("gain", "f"),
-                ("equivalent_beam_angle", "f"),
-                ("beamwidth_alongship", "f"),
-                ("beamwidth_athwartship", "f"),
-                ("angle_sensitivity_alongship", "f"),
-                ("angle_sensitivity_athwartship", "f"),
-                ("angle_offset_alongship", "f"),
-                ("angle_offset_athwartship", "f"),
-                ("pos_x", "f"),
-                ("pos_y", "f"),
-                ("pos_z", "f"),
-                ("dir_x", "f"),
-                ("dir_y", "f"),
-                ("dir_z", "f"),
-                ("pulse_length_table", "5f"),
-                ("spare1", "8s"),
-                ("gain_table", "5f"),
-                ("spare2", "8s"),
-                ("sa_correction_table", "5f"),
-                ("spare3", "8s"),
-                ("gpt_software_version", "16s"),
-                ("spare4", "28s"),
-            ],
+            "ER60": self.COMMON_KEYS,
+            "ES60": self.COMMON_KEYS,
+            "ES70": self.COMMON_KEYS,
             "MBES": [
                 ("channel_id", "128s"),
                 ("beam_type", "l"),
@@ -1408,11 +1385,11 @@ class SimradConfigParser(_SimradDatagramParser):
                 txcvr_header_values = list(txcvr_header_values_encoded)
                 for tx_idx, tx_val in enumerate(txcvr_header_values_encoded):
                     if isinstance(tx_val, bytes):
-                        txcvr_header_values[tx_idx] = tx_val.decode()
+                        txcvr_header_values[tx_idx] = tx_val.decode("latin_1")
 
                 txcvr = data["transceivers"].setdefault(txcvr_indx, {})
 
-                if _sounder_name_used in ["ER60", "ES60"]:
+                if _sounder_name_used in ["ER60", "ES60", "ES70"]:
                     for txcvr_field_indx, field in enumerate(txcvr_header_fields[:17]):
                         txcvr[field] = txcvr_header_values[txcvr_field_indx]
 
@@ -1505,7 +1482,7 @@ class SimradConfigParser(_SimradDatagramParser):
             for txcvr_indx, txcvr in list(data["transceivers"].items()):
                 txcvr_contents = []
 
-                if _sounder_name_used in ["ER60", "ES60"]:
+                if _sounder_name_used in ["ER60", "ES60", "ES70"]:
                     for field in txcvr_header_fields[:17]:
                         txcvr_contents.append(txcvr[field])
 
@@ -1666,7 +1643,7 @@ class SimradRawParser(_SimradDatagramParser):
 
             else:
                 data["power"] = np.empty((0,), dtype="int16")
-                data["angle"] = np.empty((0,), dtype="int8")
+                data["angle"] = np.empty((0, 2), dtype="int8")
 
         elif version == 3:
 
