@@ -1,3 +1,4 @@
+import datetime
 import uuid
 import warnings
 from collections import OrderedDict
@@ -13,10 +14,12 @@ from zarr.errors import GroupNotFoundError, PathNotFoundError
 if TYPE_CHECKING:
     from ..core import EngineHint, FileFormatHint, PathHint, SonarModelsHint
 
+from ..utils.coding import set_encodings
 from ..utils.io import check_file_existence, sanitize_file_path
 from ..utils.repr import HtmlTemplate
 from ..utils.uwa import calc_sound_speed
 from .convention import _get_convention
+from .convention.attrs import DEFAULT_PLATFORM_COORD_ATTRS
 
 XARRAY_ENGINE_MAP: Dict["FileFormatHint", "EngineHint"] = {
     ".nc": "netcdf4",
@@ -43,6 +46,7 @@ class EchoData:
         source_file: Optional["PathHint"] = None,
         xml_path: "PathHint" = None,
         sonar_model: "SonarModelsHint" = None,
+        open_kwargs: Dict[str, str] = None,
     ):
 
         # TODO: consider if should open datasets in init
@@ -51,6 +55,9 @@ class EchoData:
         self.storage_options: Dict[str, str] = (
             storage_options if storage_options is not None else {}
         )
+        self.open_kwargs: Dict[str, str] = (
+            open_kwargs if open_kwargs is not None else {}
+        )
         self.source_file: Optional["PathHint"] = source_file
         self.xml_path: Optional["PathHint"] = xml_path
         self.sonar_model: Optional["SonarModelsHint"] = sonar_model
@@ -137,35 +144,48 @@ class EchoData:
         """
         Computes the range of the data contained in this `EchoData` object, in meters.
 
-        This method only applies to `sonar_model`s of `"AZFP"`, `"EK60"`, and `"EK80"`.
-        If the `sonar_model` is not `"AZFP"`, `"EK60"`, or `"EK80"`, an error is raised.
+        Currently this operation is supported for the following ``sonar_model``:
+        EK60, AZFP, EK80 (see Notes below for detail).
 
         Parameters
         ----------
-        env_params: dict
-            This dictionary should contain either:
-            - `"sound_speed"`: `float`
-            - `"temperature"`, `"salinity"`, and `"pressure"`: `float`s,
-            in which case the sound speed will be calculated.
-            If the `sonar_model` is `"EK60"` or `"EK80"`, and
-            `EchoData.environment.sound_speed_indicative` exists, then this parameter
-            does not need to be specified.
+        env_params : dict
+            Environmental parameters needed for computing sonar range.
+            Users can supply `"sound speed"` directly,
+            or specify other variables that can be used to compute them,
+            including `"temperature"`, `"salinity"`, and `"pressure"`.
+
+            For EK60 and EK80 echosounders, by default echopype uses
+            environmental variables stored in the data files.
+            For AZFP echosounder, all environmental parameters need to be supplied.
+            AZFP echosounders typically are equipped with an internal temperature
+            sensor, and some are equipped with a pressure sensor, but automatically
+            using these pressure data is not currently supported.
+
         azfp_cal_type : {"Sv", "Sp"}, optional
+
             - `"Sv"` for calculating volume backscattering strength
             - `"Sp"` for calculating point backscattering strength.
-            This parameter is only used if `sonar_model` is `"AZFP"`,
-            and in that case it must be specified.
+
+            This parameter needs to be specified for data from the AZFP echosounder,
+            due to a difference in computing range for Sv and Sp.
+
         ek_waveform_mode : {"CW", "BB"}, optional
-            - `"CW"` for CW-mode samples, either recorded as complex or power samples
-            - `"BB"` for BB-mode samples, recorded as complex samples
-            This parameter is only used if `sonar_model` is `"EK60"` or `"EK80"`,
-            and in those cases it must be specified.
+            Type of transmit waveform.
+            Required only for data from the EK80 echosounder.
+
+            - `"CW"` for narrowband transmission,
+              returned echoes recorded either as complex or power/angle samples
+            - `"BB"` for broadband transmission,
+              returned echoes recorded as complex samples
+
         ek_encode_mode : {"complex", "power"}, optional
-            For EK80 data, range can be computed from complex or power samples.
-            The type of sample used can be specified with this parameter.
-            - `"complex"` to use complex samples
-            - `"power"` to use power samples
-            This parameter is only used if `sonar_model` is `"EK80"`.
+            Type of encoded return echo data.
+            Required only for data from the EK80 echosounder.
+
+            - `"complex"` for complex samples
+            - `"power"` for power/angle samples, only allowed when
+              the echosounder is configured for narrowband transmission
 
         Returns
         -------
@@ -176,17 +196,31 @@ class EchoData:
         ------
         ValueError
             - When `sonar_model` is `"AZFP"` but `azfp_cal_type` is not specified or is `None`.
-            - When `sonar_model` is `"EK60"` or `"EK80"` but `ek_waveform_mode`
-            is not specified or is `None`.
-            - When `sonar_model` is `"EK60"` but `waveform_mode` is `"BB"` (EK60 cannot have
-            broadband samples).
+            - When `sonar_model` is `"EK80"` but `ek_waveform_mode` is not specified or is `None`.
+            - When `sonar_model` is `"EK60"` but `waveform_mode` is `"BB"`
             - When `sonar_model` is `"AZFP"` and `env_params` does not contain
-            either `"sound_speed"` or all of `"temperature"`, `"salinity"`, and `"pressure"`.
+              either `"sound_speed"` or all of `"temperature"`, `"salinity"`, and `"pressure"`.
             - When `sonar_model` is `"EK60"` or `"EK80"`,
-            EchoData.environment.sound_speed_indicative does not exist,
-            and `env_params` does not contain either `"sound_speed"` or all of `"temperature"`,
-            `"salinity"`, and `"pressure"`.
+              EchoData.environment.sound_speed_indicative does not exist,
+              and `env_params` does not contain either `"sound_speed"`
+              or all of `"temperature"`, `"salinity"`, and `"pressure"`.
             - When `sonar_model` is not `"AZFP"`, `"EK60"`, or `"EK80"`.
+
+        Notes
+        -----
+        The EK80 echosounder can be configured to transmit
+        either broadband (``waveform_mode="BB"``)
+        or narrowband (``waveform_mode="CW"``) signals.
+        When transmitting in broadband mode, the returned echoes are
+        encoded as complex samples (``encode_mode="complex"``).
+        When transmitting in narrowband mode, the returned echoes can be encoded
+        either as complex samples (``encode_mode="complex"``)
+        or as power/angle combinations (``encode_mode="power"``) in a format
+        similar to those recorded by EK60 echosounders.
+
+        For AZFP echosounder, the returned range is duplicated along `ping_time`
+        to conform with outputs from other echosounders, even though within each data
+        file the range is held constant.
         """
 
         def squeeze_non_scalar(n):
@@ -217,6 +251,7 @@ class EchoData:
                 "or in EchoData.environment.sound_speed_indicative for EK60 and EK80 sonar models"
             )
 
+        # AZFP
         if self.sonar_model == "AZFP":
             cal_type = azfp_cal_type
             if cal_type is None:
@@ -253,18 +288,37 @@ class EchoData:
             )
             range_meter.name = "range"  # add name to facilitate xr.merge
 
+            # duplicate range for all ping_times for consistency with EK case
+            # if it is not indexed by ping_time then echopype.preprocess.compute_MVBS will fail
+            #   because it expects the range included in the Sv/Sp dataset
+            #   to be indexed by ping_time
+            range_meter = range_meter.expand_dims(
+                {"ping_time": self.beam["ping_time"]}, axis=1
+            )
+
             return range_meter
+
+        # EK
         elif self.sonar_model in ("EK60", "EK80"):
             waveform_mode = ek_waveform_mode
             encode_mode = ek_encode_mode
 
-            if self.sonar_model == "EK60" and waveform_mode == "BB":
-                raise ValueError("EK60 cannot have BB samples")
+            # EK60 can only be CW mode
+            if self.sonar_model == "EK60":
+                if waveform_mode is None:
+                    waveform_mode = "CW"  # default to CW mode
+                elif waveform_mode != "CW":
+                    raise ValueError("EK60 must have CW samples")
 
-            if waveform_mode is None:
+            # EK80 needs waveform_mode specification
+            if self.sonar_model == "EK80" and waveform_mode is None:
                 raise ValueError(
-                    "ek_waveform_mode must be specified when sonar_model is EK60 or EK80"
+                    "ek_waveform_mode must be specified when sonar_model is EK80"
                 )
+
+            # TVG correction factor changes depending when the echo recording starts
+            # wrt when the transmit signal is sent out.
+            # This implementation is different for EK60 and EK80.
             tvg_correction_factor = TVG_CORRECTION_FACTOR[self.sonar_model]
 
             if waveform_mode == "CW":
@@ -273,6 +327,8 @@ class EchoData:
                     and encode_mode == "power"
                     and self.beam_power is not None
                 ):
+                    # if both CW and BB exist and beam_power group is not empty
+                    # this means that CW is recorded in power/angle mode
                     beam = self.beam_power
                 else:
                     beam = self.beam
@@ -307,12 +363,19 @@ class EchoData:
             range_meter.name = "range"  # add name to facilitate xr.merge
 
             return range_meter
+
+        # OTHERS
         else:
             raise ValueError(
-                "this method only supports sonar_model values of AZFP, EK60, and EK80"
+                "this method only supports the following sonar_model: AZFP, EK60, and EK80"
             )
 
-    def update_platform(self, extra_platform_data: xr.Dataset, time_dim="time"):
+    def update_platform(
+        self,
+        extra_platform_data: xr.Dataset,
+        time_dim="time",
+        extra_platform_data_file_name=None,
+    ):
         """
         Updates the `EchoData.platform` group with additional external platform data.
 
@@ -327,7 +390,8 @@ class EchoData:
             - `"latitude"`
             - `"longitude"`
             - `"water_level"`
-        The data inserted into the Platform group will be indexed by a dimension named `"time2"`.
+        The data inserted into the Platform group will be indexed by a dimension named
+        `"location_time"`.
 
         Parameters
         ----------
@@ -337,17 +401,17 @@ class EchoData:
         time_dim: str, default="time"
             The name of the time dimension in `extra_platform_data`; used for extracting
             data from `extra_platform_data`.
+        extra_platform_data_file_name: str, default=None
+            File name for source of extra platform data, if read from a file
 
         Examples
         --------
         >>> ed = echopype.open_raw(raw_file, "EK60")
         >>> extra_platform_data = xr.open_dataset(extra_platform_data_file)
-        >>> ed.update_platform(extra_platform_data)
+        >>> ed.update_platform(extra_platform_data,
+        >>>         extra_platform_data_file_name=extra_platform_data_file)
         """
 
-        # # only take data during ping times
-        # start_time, end_time = min(self.beam["ping_time"]), max(self.beam["ping_time"])
-
         # Handle data stored as a CF Trajectory Discrete Sampling Geometry
         # http://cfconventions.org/Data/cf-conventions/cf-conventions-1.8/cf-conventions.html#trajectory-data
         # The Saildrone sample data file follows this convention
@@ -370,23 +434,74 @@ class EchoData:
             extra_platform_data = extra_platform_data.drop_vars(trajectory_var)
             extra_platform_data = extra_platform_data.swap_dims({"obs": time_dim})
 
-        platform = self.platform.assign_coords(
-            time2=extra_platform_data[time_dim].values
+        # clip incoming time to 1 less than min of EchoData.beam["ping_time"] and
+        #   1 greater than max of EchoData.beam["ping_time"]
+        # account for unsorted external time by checking whether each time value is between
+        #   min and max ping_time instead of finding the 2 external times corresponding to the
+        #   min and max ping_time and taking all the times between those indices
+        sorted_external_time = extra_platform_data[time_dim].data
+        sorted_external_time.sort()
+        # fmt: off
+        min_index = max(
+            np.searchsorted(
+                sorted_external_time, self.beam["ping_time"].min(), side="left"
+            ) - 1,
+            0,
+        )
+        # fmt: on
+        max_index = min(
+            np.searchsorted(
+                sorted_external_time, self.beam["ping_time"].max(), side="right"
+            ),
+            len(sorted_external_time) - 1,
+        )
+        extra_platform_data = extra_platform_data.sel(
+            {
+                time_dim: np.logical_and(
+                    sorted_external_time[min_index] <= extra_platform_data[time_dim],
+                    extra_platform_data[time_dim] <= sorted_external_time[max_index],
+                )
+            }
         )
-        platform["time2"].attrs[
-            "long_name"
-        ] = "time dimension for external platform data"
 
+        platform = self.platform
+        platform_vars_attrs = {var: platform[var].attrs for var in platform.variables}
+        platform = platform.drop_dims(["location_time"], errors="ignore")
+        # drop_dims is also dropping latitude, longitude and sentence_type why?
+        platform = platform.assign_coords(
+            location_time=extra_platform_data[time_dim].values
+        )
+        history_attr = (
+            f"{datetime.datetime.utcnow()} +00:00. Added from external platform data"
+        )
+        if extra_platform_data_file_name:
+            history_attr += ", from file " + extra_platform_data_file_name
+        location_time_attrs = {
+            **DEFAULT_PLATFORM_COORD_ATTRS["location_time"],
+            **{"history": history_attr},
+        }
+        platform["location_time"] = platform["location_time"].assign_attrs(
+            **location_time_attrs
+        )
+
+        dropped_vars_target = [
+            "pitch",
+            "roll",
+            "heave",
+            "latitude",
+            "longitude",
+            "water_level",
+        ]
         dropped_vars = []
-        for var in ["pitch", "roll", "heave", "latitude", "longitude", "water_level"]:
+        for var in dropped_vars_target:
             if var in platform and (~platform[var].isnull()).all():
                 dropped_vars.append(var)
         if len(dropped_vars) > 0:
             warnings.warn(
-                f"some variables in the original Platform group will be overwritten: {', '.join(dropped_vars)}"  # noqa
+                f"Some variables in the original Platform group will be overwritten: {', '.join(dropped_vars)}"  # noqa
             )
         platform = platform.drop_vars(
-            ["pitch", "roll", "heave", "latitude", "longitude", "water_level"],
+            dropped_vars_target,
             errors="ignore",
         )
 
@@ -398,10 +513,10 @@ class EchoData:
                     return mapping[key].data
             return default
 
-        self.platform = platform.update(
+        platform = platform.update(
             {
                 "pitch": (
-                    "time2",
+                    "location_time",
                     mapping_search_variable(
                         extra_platform_data,
                         ["pitch", "PITCH"],
@@ -409,7 +524,7 @@ class EchoData:
                     ),
                 ),
                 "roll": (
-                    "time2",
+                    "location_time",
                     mapping_search_variable(
                         extra_platform_data,
                         ["roll", "ROLL"],
@@ -417,7 +532,7 @@ class EchoData:
                     ),
                 ),
                 "heave": (
-                    "time2",
+                    "location_time",
                     mapping_search_variable(
                         extra_platform_data,
                         ["heave", "HEAVE"],
@@ -425,7 +540,7 @@ class EchoData:
                     ),
                 ),
                 "latitude": (
-                    "time2",
+                    "location_time",
                     mapping_search_variable(
                         extra_platform_data,
                         ["lat", "latitude", "LATITUDE"],
@@ -433,7 +548,7 @@ class EchoData:
                     ),
                 ),
                 "longitude": (
-                    "time2",
+                    "location_time",
                     mapping_search_variable(
                         extra_platform_data,
                         ["lon", "longitude", "LONGITUDE"],
@@ -441,7 +556,7 @@ class EchoData:
                     ),
                 ),
                 "water_level": (
-                    "time2",
+                    "location_time",
                     mapping_search_variable(
                         extra_platform_data,
                         ["water_level", "WATER_LEVEL"],
@@ -450,6 +565,10 @@ class EchoData:
                 ),
             }
         )
+        for var in dropped_vars_target:
+            platform[var] = platform[var].assign_attrs(**platform_vars_attrs[var])
+
+        self.platform = set_encodings(platform)
 
     @classmethod
     def _load_convert(cls, convert_obj):
@@ -507,7 +626,9 @@ class EchoData:
     def _load_group(self, filepath: "PathHint", group: Optional[str] = None):
         """Loads each echodata group"""
         suffix = self._check_suffix(filepath)
-        return xr.open_dataset(filepath, group=group, engine=XARRAY_ENGINE_MAP[suffix])
+        return xr.open_dataset(
+            filepath, group=group, engine=XARRAY_ENGINE_MAP[suffix], **self.open_kwargs
+        )
 
     def to_netcdf(self, save_path: Optional["PathHint"] = None, **kwargs):
         """Save content of EchoData to netCDF.
